--Legal Notice: (C)2009 Altera Corporation. All rights reserved.  Your
--use of Altera Corporation's design tools, logic functions and other
--software and tools, and its AMPP partner logic functions, and any
--output files any of the foregoing (including device programming or
--simulation files), and any associated documentation or information are
--expressly subject to the terms and conditions of the Altera Program
--License Subscription Agreement or other applicable license agreement,
--including, without limitation, that your use is for the sole purpose
--of programming logic devices manufactured by Altera and sold by Altera
--or its authorized distributors.  Please refer to the applicable
--agreement for further details.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity pfl_control is
	port (
		fpga_conf_done : in std_logic;
		fpga_nstatus : in std_logic;
		clk : in std_logic;
		load_image : in std_logic;
		image_sel_in : in std_logic;
		factory_user : in std_logic;
		
		fsm_d : inout std_logic_vector(15 downto 0);
		fsm_a : inout std_logic_vector(24 downto 0);
		flash_cen : out std_logic;
		flash_wen : out std_logic;
		flash_oen : out std_logic;
		flash_clk : out std_logic;
		flash_advn : out std_logic;
		flash_resetn : out std_logic;
		
		fpga_config_d : out std_logic_vector(7 downto 0);
		fpga_dclk : out std_logic;
		fpga_nconfig : out std_logic;
		
		max_led : out std_logic;
		
		image_led : out std_logic_vector(2 downto 0);
		
		srst : in std_logic;
		reset_n : in std_logic;
		
		pfl_pgm : in std_logic_vector(2 downto 0);
		
		fsd_req : out std_logic;
		pfl_en_out : out std_logic;
		fpga_pgm_out : out std_logic_vector(2 downto 0)
--		max_factory : out std_logic;
		
--		clk_pls_p: in std_logic -- from 50MHz, it will be 48.828KHz or 20.480us
	);
end pfl_control;


architecture rtl of pfl_control is
	signal fl_access_req : std_logic;
	signal pfl_en, pfl_en_int : std_logic;
	signal reconfigure_n : std_logic;
	signal res_in : std_logic;
	signal pls_out : std_logic;
	signal fpga_pgm : std_logic_vector(2 downto 0);
	signal image_led_int : std_logic_vector(2 downto 0);
	
	signal pfl_data : std_logic_vector(7 downto 0);
	signal pfl_dclk : std_logic;
	signal pfl_nConfig : std_logic;
	signal fppOvj_data : std_logic_vector(7 downto 0);
	signal fppOvj_dclk : std_logic;
	signal fppOvj_nConfig : std_logic;

	COMPONENT p3 PORT(
		fpga_conf_done		: IN STD_LOGIC ;
		fpga_nstatus		: IN STD_LOGIC ;
		fpga_pgm		: IN STD_LOGIC_VECTOR (2 DOWNTO 0);
		pfl_clk		: IN STD_LOGIC ;
		pfl_flash_access_granted		: IN STD_LOGIC ;
		pfl_nreconfigure		: IN STD_LOGIC  := '1';
		pfl_nreset		: IN STD_LOGIC ;
		flash_addr		: OUT STD_LOGIC_VECTOR (24 DOWNTO 0);
--		flash_clk		: OUT STD_LOGIC ;
		flash_data		: INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
--		flash_nadv		: OUT STD_LOGIC ;
		flash_nce		: OUT STD_LOGIC ;
		flash_noe		: OUT STD_LOGIC ;
--		flash_nreset		: OUT STD_LOGIC ;
		flash_nwe		: OUT STD_LOGIC ;
		fpga_data		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		fpga_dclk		: OUT STD_LOGIC ;
		fpga_nconfig		: OUT STD_LOGIC ;
		pfl_flash_access_request		: OUT STD_LOGIC 
	);
	END COMPONENT;
	
	COMPONENT wdt port (
		clk: in std_logic;
		reset_n : in std_logic;
		fpga_conf_done : in std_logic;
		clk_pls_p: in std_logic; -- from 50MHz, it will be 48.828KHz or 20.480us
		reset: out std_logic
	);
	end COMPONENT;
	
	COMPONENT reset_pls port (
			clk: in std_logic;
			pls_out: out std_logic
		);
	end COMPONENT;

	COMPONENT image_sel2 port (
			reset_n : in std_logic;
			clk: in std_logic;
			clk_pls_p: in std_logic; -- from 50MHz, it will be 48.828KHz or 20.480us
			
			image_sel : in std_logic;
			factory_user : in std_logic;
			pfl_en : in std_logic;
			pfl_pgm : in std_logic_vector(2 downto 0);
			
			image_led : out std_logic_vector(2 downto 0);
			max_factory : out std_logic;
			fpga_pgm : out std_logic_vector(2 downto 0)
		);
	end COMPONENT;
	
	signal conf_done1, conf_done2 : std_logic;



--This is for FPPoVJ	
	COMPONENT fppOvj port (
		system_clk : in std_logic;
		reset_n : in std_logic;
		fpga_nstatus : IN STD_LOGIC;
		data_out_port : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		clk_out_port : out std_logic;
		nConfig_out_port : out std_logic
    );
	end COMPONENT;

--This is for FPPoVJ	
	COMPONENT conf_sel port (
		data_in_a : in STD_LOGIC_VECTOR (7 DOWNTO 0);
		clk_in_a : in std_logic;
		nConfig_in_a : in std_logic;
		data_in_b : in STD_LOGIC_VECTOR (7 DOWNTO 0);
		clk_in_b : in std_logic;
		nConfig_in_b : in std_logic;			
		sel : in std_logic;
		data_out : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		clk_out : out std_logic;
		nConfig_out : out std_logic
	);
	end COMPONENT;
	
	signal max_factory : std_logic; -- just for dummy

	COMPONENT slow_clk_gen is
		port (
			clk: in std_logic;
			reset : in std_logic;
			clk_pls_n: out std_logic;
			clk_pls_p: out std_logic;
			
			slow_clk : out std_logic -- from 50MHz, it will be 48.828KHz or 20.480us
		);
	end COMPONENT;
	signal clk_pls_n: std_logic;
	signal clk_pls_p: std_logic;
	signal slow_clk : std_logic;
	
begin

	slow_clk_gen_inst : slow_clk_gen port map(
			clk => clk,
			reset => reset_n,
			clk_pls_n => clk_pls_n,
			clk_pls_p => clk_pls_p,			
			slow_clk => slow_clk
		);

--	p2_inst : P2  port map(
--		fpga_conf_done, fpga_nstatus, fpga_pgm, clk, fl_access_req, reconfigure_n, pfl_en,
--		fsm_a, fsm_d, flash_cen, flash_oen, flash_wen,
--		pfl_data, pfl_dclk, pfl_nConfig, fl_access_req);
	p3_inst : P3  port map(
		fpga_conf_done, fpga_nstatus, pfl_pgm, clk, fl_access_req and pfl_en, reconfigure_n, pfl_en,
		fsm_a, fsm_d, flash_cen, flash_oen, flash_wen,
		pfl_data, pfl_dclk, pfl_nConfig, fl_access_req);
		
		
		
			flash_clk <= '0';
			flash_advn <= '0';
			flash_resetn <= '1';


	res_in <= reconfigure_n and reset_n;			
	wdt_inst : wdt port map(clk, res_in, fpga_conf_done, clk_pls_p, pfl_en);					
	reset_pls_inst : reset_pls port map(clk, pls_out);
	
	reconfigure_n <= load_image and pls_out and srst;  

	
	image_sel_inst : image_sel2 port map(
			reset_n, clk, clk_pls_p, image_sel_in, factory_user, pfl_en, pfl_pgm,
			image_led_int, max_factory, fpga_pgm);

	max_led <= not fpga_conf_done;
	
	fsd_req <= fl_access_req;
	fpga_pgm_out <= fpga_pgm;
	pfl_en_out <= pfl_en;
	
	image_led <= image_led_int;


	fppOvj_inst : fppOvj port map(clk, reset_n, fpga_nstatus, fppOvj_data, fppOvj_dclk, fppOvj_nConfig);
	conf_sel_inst : conf_sel port map(
		pfl_data, pfl_dclk, pfl_nConfig,
		fppOvj_data, fppOvj_dclk, fppOvj_nConfig,
		pfl_en,
		fpga_config_d, fpga_dclk, fpga_nconfig
	);


end rtl;
