library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity power_calc is

	port
	(
	-- inputs
		--globals
		reset_n: in std_logic;
		clk: in std_logic;
		
		--adc data interface
		dclk: in std_logic;
		diff: in std_logic_vector(23 downto 0);
		sgl_in : in std_logic;
		
		mux_channel : in std_logic_vector(3 downto 0);
	
	-- outputs
		ave_v_out : out std_logic_vector(15 downto 0);
		ave_a_out : out std_logic_vector(15 downto 0);
		ave_w_out : out std_logic_vector(15 downto 0);
		max_v_out : out std_logic_vector(15 downto 0);
		max_a_out : out std_logic_vector(15 downto 0);
		max_w_out : out std_logic_vector(15 downto 0);
		min_v_out : out std_logic_vector(15 downto 0);
		min_a_out : out std_logic_vector(15 downto 0);
		min_w_out : out std_logic_vector(15 downto 0)
	);

end power_calc;


architecture rtl of power_calc is
	signal count : integer range 0 to 118;
	signal protect : integer range 0 to 4;
	signal mul_en, dummy, stop, change : std_logic;
	signal data_in1 : std_logic_vector(47 downto 0);
	signal data_in2 : std_logic_vector(23 downto 0);
	signal mul_dout : std_logic_vector(71 downto 0);
	signal scaler_squared_div_009_squared2: std_logic_vector(21 downto 0);
--	signal ave_1, ave_2, ave_3, ave_4, ave_5, ave_6, ave_7 : std_logic_vector(15 downto 0) := (others => '0');
--	signal ave_0 : std_logic_vector(18 downto 0) := (others => '0');
	signal ave_v, ave_a, ave_w : std_logic_vector(15 downto 0) := (others => '0');
	signal min_v, min_a, min_w, max_v, max_a, max_w : std_logic_vector(15 downto 0) := (others => '0');
	signal keep : std_logic_vector(15 downto 0) := (others => '0');
	signal dclk_i, en  : std_logic;
	
	component mul_48x24
		port(
			reset : in std_logic;
			clk : in std_logic;
			data_in1: in std_logic_vector(47 downto 0);
			data_in2: in std_logic_vector(23 downto 0);
			data_out: out std_logic_vector(71 downto 0)
		);
	end component;

begin

	U1: mul_48x24 port map(mul_en, clk, data_in1, data_in2, mul_dout);


-- waiting for dividend and divisor to be setup
	process(reset_n, clk)begin
		if(reset_n = '0')then
			dclk_i <= '0';
			en <= '0';
		elsif(clk'event and clk = '1')then
			dclk_i <= not dclk;
			en <= dclk_i;
		end if;
	end process;


	process(reset_n, clk)begin
		if(reset_n = '0')then
			count <= 0;
			protect <= 0;
		elsif(clk'event and clk = '1')then
			if(en = '0' or stop = '1')then
				count <= 0;
			elsif(count = 89)then
				if(protect < 4)then
					protect <= protect + 1;
				else
					protect <= 4;
				end if;
				count <= count + 1;
			elsif(count = 118)then
				count <= count;
			else
				count <= count + 1;
			end if;
		end if;
	end process;
	
	process(reset_n, clk)begin
		if(reset_n = '0')then
			keep <= (others => '0');
--			ave_0 <= (others => '0');
--			ave_1 <= (others => '0');
--			ave_2 <= (others => '0');
--			ave_3 <= (others => '0');
--			ave_4 <= (others => '0');
--			ave_5 <= (others => '0');
--			ave_6 <= (others => '0');
--			ave_7 <= (others => '0');
			ave_a <= (others => '0');
			ave_w <= (others => '0');
			min_v <= (others => '1');
			min_a <= (others => '1');
			min_w <= (others => '1');
			max_v <= (others => '0');
			max_a <= (others => '0');
			max_w <= (others => '0');
		elsif(clk'event and clk = '1')then
			if(en = '0')then
				mul_en <= '0';
				data_in1 <= (others => '0');
				data_in2 <= (others => '0');
				stop <= '0';
			elsif(count = 1)then
			elsif(count = 2)then
				data_in1(23 downto 0) <= diff(23 downto 0);
				if(sgl_in = '1')then
					--data_in2(23 downto 0) <= "000000000000110011100100"; --3300
					--data_in2(23 downto 0) <= "000000000000111100111100"; --3900
					--data_in2(23 downto 0) <= "000000000001001110001000"; -- 5000
					  data_in2(23 downto 0) <= "000000000001010010110100"; -- 5300
				else
					--data_in2(20 downto 0) <= "100001100100011100000"; -- 0x10C8E0 = 3.3/0.003 * 1000
					--data_in2(20 downto 0) <= "100111101011000100000"; -- 3.9/0.003 * 1000
					--data_in2(20 downto 0) <= "110010110111001101010"; -- 5/0.003 * 1000
					if(mux_channel = "1010")then
--						data_in2(23 downto 0) <= "010011000100101101000000"; -- 5/0.001 * 1000
						data_in2(23 downto 0) <= "010100001101111100100000"; -- 5.3/0.001 * 1000
					else
--						data_in2(20 downto 0) <= "010000111101000100011"; -- 5/0.009 * 1000
						data_in2(20 downto 0) <= "010001111110001011000"; -- 5.3/0.009 * 1000
						data_in2(23 downto 21) <= (others => '0');
					end if;
				end if;
			elsif(count = 3)then
				mul_en <= '1';
			elsif(count = 30)then
				keep <= mul_dout(38 downto 23); -- shift 23bits
				if(sgl_in = '1')then
					-- re-use data_in1
					--data_in2(8 downto 0) <= "110100110"; -- 0x1A6 = 3.3 * 128
					--data_in2(8 downto 0) <= "111110011"; -- 3.9*128
					--data_in2(23 downto 9) <= (others => '0');
--					data_in2(9 downto 0) <= "1010000000"; -- 0x280 = 5.0 * 128
					data_in2(9 downto 0) <= "1010100110"; -- 5.3 * 128
					data_in2(23 downto 10) <= (others => '0');
				else
					--data_in2(11 downto 0) <= "110011100100"; -- 3300
					--data_in2(23 downto 12) <= (others => '0');
--					data_in2(12 downto 0) <= "1001110001000"; -- 5000
					data_in2(12 downto 0) <= "1010010110100"; -- 5300
					data_in2(23 downto 13) <= (others => '0');
				end if;
				-- re-use the data_in1
				mul_en <= '0';
			elsif(count = 31)then
				if(sgl_in = '1')then
					ave_v <= keep;
					if(protect = 4)then
						if(max_v < keep)then
							max_v <= keep;
						end if;
						if(min_v > keep)then
							min_v <= keep;
						end if;
					end if;
				else
					ave_a <= keep;
					if(protect = 4)then
						if(max_a < keep)then
							max_a <= keep;
						end if;
						if(min_a > keep)then
							min_a <= keep;
						end if;
					end if;
				end if;
			elsif(count = 32)then
				mul_en <= '1';
			elsif(count = 58)then
				if(sgl_in = '1')then
					data_in1(47 downto 0) <= mul_dout(47 downto 0);
					data_in2(15 downto 0) <= ave_a(15 downto 0);
					data_in2(23 downto 16) <= (others => '0');
				end if;
				mul_en <= '0';
			elsif(count = 60)then
				mul_en <= '1';
			elsif(count = 86)then
				keep <= mul_dout(45 downto 30);
				mul_en <= '0';
			elsif(count = 87)then
				if(sgl_in = '1')then
					ave_w <= keep;
					if(protect = 4)then
						if(max_w < keep)then
							max_w <= keep;
						end if;
						if(min_w > keep and keep > 0)then
							min_w <= keep;
						end if;
					end if;
				end if;
			elsif(count = 89)then
			elsif(count > 89 and dummy = '1')then
				stop <= '1';			
			end if;

		end if;
	end process;


	-- dummy to prevent circuit drop during compilation
	process(keep)begin
		if(keep /=0)then
			dummy <= '1';
		else
			dummy <= '0';
		end if;
	end process;
	

ave_v_out <= ave_v;
ave_a_out <= ave_a;
ave_w_out <= ave_w;
max_v_out <= max_v;
max_a_out <= max_a;
max_w_out <= max_w;
min_v_out <= min_v;
min_a_out <= min_a;
min_w_out <= min_w;
	
end rtl;