library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity system_max is
port(
	max_resetn : in std_logic;
	clk : in std_logic;

	fsm_d : inout std_logic_vector(31 downto 0);
	fsm_a : inout std_logic_vector(25 downto 0);

	-- flash interface
	flash_clk : out std_logic;
	flash_advn : out std_logic;
	flash_cen : out std_logic;
	flash_oen : out std_logic;
	--flash_rdy_bsyn : in std_logic;
	flash_resetn : out std_logic;
	flash_wen : out std_logic;

	-- fpga/max interface
	max2_ben : in std_logic_vector(3 downto 0);
	max2_clk : in std_logic;
	max2_csn : in std_logic;
	max2_oen : in std_logic;
	max2_wen : in std_logic;

	-- fpga conifg interface
	fpga_conf_done : in std_logic;
	fpga_config_d : out std_logic_vector(7 downto 0);
	fpga_dclk : out std_logic;
	fpga_nconfig : out std_logic;
	fpga_nstatus : in std_logic;

	-- pwoer monitor interface
	csense_sdo : in std_logic;
	csense_sck : out std_logic;
	csense_sdi : out std_logic;
	csense_csn : out std_logic_vector(1 downto 0);
	csense_adc_f0 : out std_logic;

	
	-- CLK control
	clk_sel : in std_logic;
	clk_enable : in std_logic;
	clk155_oe : out std_logic;
	clk1_ce : out std_logic;
	clk1_od : out std_logic_vector(2 downto 0);
	--clk1_os : out std_logic_vector(1 downto 0);
	clk1_pr : out std_logic_vector(1 downto 0);
	clk1_rstn : out std_logic;
	clk2_ce : out std_logic;
	clk2_od : out std_logic_vector(2 downto 0);
	--clk2_os : out std_logic_vector(1 downto 0);
	clk2_pr : out std_logic_vector(1 downto 0);
	clk2_rstn : out std_logic;


	-- configuration related
	usb_led : in std_logic;
	max_load : out std_logic;
	max_error : out std_logic;
	max_led : out std_logic;
	factory_user : in std_logic;
	image_sel : in std_logic;
	load_image : in std_logic;
	image_led : out std_logic_vector(2 downto 0);


--lcd_pwrmon
--clk100_cpld
--max_dip : in std_logic_vector(2 downto 0);
--ddr2_scl
--ddr2_sda
--ep_clk
--ep_cs
--ep_di
--ep_do
	hsma_psnt_n : in std_logic;
--	hsmb_psnt_n : in std_logic;   --no hsmb in A2GX125
--usb_disablen

	sram_mode : out std_logic;
	sram_zz : out std_logic
);
end system_max;

architecture rtl of system_max is

	constant MAX_VER  : std_logic_vector(7 downto 0) := x"0D";

	signal fsd_req : std_logic;
	
	
	component reset_generator is
		port (
			clk: in std_logic;
			reset_in : in std_logic;
	--		reset_vj : in std_logic;
			reset: out std_logic
		);
	end component;
	signal reset_n : std_logic;
	
	component power is
		port (
			reset_n: in std_logic;
			reset_vj : in std_logic;
			clk : in std_logic;
			csense_sdo: in std_logic;
	--		pmon_railsel : in std_logic_vector(3 downto 0);
	--		gui_control : in std_logic;
			control_gui : in std_logic_vector(7 downto 0);
			pgm : in std_logic_vector(3 downto 0);

			csense_sck : out std_logic;
			sense_ce0 : out std_logic;
			sense_ce1 : out std_logic;
			csense_sdi : out std_logic;
			ave_v : out std_logic_vector(15 downto 0);
			ave_a : out std_logic_vector(15 downto 0);
			ave_w : out std_logic_vector(15 downto 0);
			max_v : out std_logic_vector(15 downto 0);
			max_a : out std_logic_vector(15 downto 0);
			max_w : out std_logic_vector(15 downto 0);
			min_v : out std_logic_vector(15 downto 0);
			min_a : out std_logic_vector(15 downto 0);
			min_w : out std_logic_vector(15 downto 0);
			rail_sel_out : out std_logic_vector(3 downto 0);
			adc_code_mon : out std_logic_vector(23 downto 0);
			sgl_mon : out std_logic;
			power_disable : in std_logic  -- 0: enable  1:disable and output hi-Z

		);
	end component;
	signal reset_vj : std_logic;
	signal control_gui : std_logic_vector(7 downto 0);
	signal pgm : std_logic_vector(3 downto 0);


	signal ave_v : std_logic_vector(15 downto 0);
	signal ave_a : std_logic_vector(15 downto 0);
	signal ave_w : std_logic_vector(15 downto 0);
	signal max_v : std_logic_vector(15 downto 0);
	signal max_a : std_logic_vector(15 downto 0);
	signal max_w : std_logic_vector(15 downto 0);
	signal min_v : std_logic_vector(15 downto 0);
	signal min_a : std_logic_vector(15 downto 0);
	signal min_w : std_logic_vector(15 downto 0);
	signal rail_sel_out : std_logic_vector(3 downto 0);

	component vj_block is
		port (
			reset_n: in std_logic;
			ave_v, ave_a, ave_w : in std_logic_vector(15 downto 0);
			max_v, max_a, max_w : in std_logic_vector(15 downto 0);
			min_v, min_a, min_w : in std_logic_vector(15 downto 0);
			temp_data_r : in std_logic_vector(15 downto 0);
			temp_data_l : in std_logic_vector(15 downto 0);
			volt_data : in std_logic_vector(15 downto 0);
			pmon_rails : in std_logic_vector(3 downto 0);
	--		remote_local_sel : in std_logic;
	--		power_temp_sel : in std_logic;
	--		gui_control : in std_logic;

			control_gui : out std_logic_vector(7 downto 0);
			reset_vj: out std_logic;
			MAX_VER : in std_logic_vector(7 downto 0)
		);
	end component;
	signal temp_data_r : std_logic_vector(15 downto 0);
	signal temp_data_l : std_logic_vector(15 downto 0);
	signal volt_data : std_logic_vector(15 downto 0);
	signal pmon_rails : std_logic_vector(3 downto 0);


	component fpga_interface is
		port
		(
			--Global signals
			reset_n: in std_logic;
			fsm_clk : in std_logic;
			
			--FPGA interface
			max_csn: in std_logic;
			max_wen: in std_logic;
			max_oen: in std_logic;
			address: in std_logic_vector(4 downto 0);
			data: inout std_logic_vector(15 downto 0);
			max_ben : in std_logic_vector(3 downto 0); -- byte write enable signal
			
			--dipswitch/PFL interface
			dsw_pagesel: in std_logic_vector(2 downto 0);
			pfl_pagesel: out std_logic_vector(2 downto 0);
			srst_out : out std_logic;
			
			-- power and temp data
			ave_v, ave_a, ave_w : in std_logic_vector(15 downto 0);
			max_v, max_a, max_w : in std_logic_vector(15 downto 0);
			min_v, min_a, min_w : in std_logic_vector(15 downto 0);
			temp_data_r : in std_logic_vector(15 downto 0);
			temp_data_l : in std_logic_vector(15 downto 0);
			volt_data : in std_logic_vector(15 downto 0);
			fsd_req : in std_logic;
			
			-- clk control
			clk155_en : out std_logic;
			clk1_ce : out std_logic;
			clk2_ce : out std_logic;
			clk_sel : in std_logic;
			clk_enable : in std_logic;
			
			-- board setting
			sram_mode : out std_logic;
			sram_zz : out std_logic;
			hsma_orsntn : in std_logic;
--			hsmb_orsntn : in std_logic;		--no hsmb in A2GX125						
			MAX_VER : in std_logic_vector(7 downto 0);
			
			-- OSC control
			CLK1_PR : out std_logic_vector(1 downto 0);
			CLK1_OD : out std_logic_vector(2 downto 0);
			CLK2_PR : out std_logic_vector(1 downto 0);
			CLK2_OD : out std_logic_vector(2 downto 0);
			clk1_rstn : out std_logic;
			clk2_rstn : out std_logic;
			
			system_clk : in std_logic
					
		);
	end component;
	signal fpga_pgm : std_logic_vector(2 downto 0);	
	signal pfl_pgm : std_logic_vector(2 downto 0);
	signal srst : std_logic;

	component err_led is
		port 
		(
			reset_n : in std_logic;
			clk : in std_logic;
			pfl_en : in std_logic;
			conf_done : in std_logic;
			error_led: out std_logic
		);
	end component;	
	
	component pfl_control is
		port (
			fpga_conf_done : in std_logic;
			fpga_nstatus : in std_logic;
			clk : in std_logic;
			load_image : in std_logic;
			image_sel_in : in std_logic;
			factory_user : in std_logic;
			
			fsm_d : inout std_logic_vector(15 downto 0);
			fsm_a : inout std_logic_vector(24 downto 0);
			flash_cen : out std_logic;
			flash_wen : out std_logic;
			flash_oen : out std_logic;
			flash_clk : out std_logic;
			flash_advn : out std_logic;
			flash_resetn : out std_logic;
			
			fpga_config_d : out std_logic_vector(7 downto 0);
			fpga_dclk : out std_logic;
			fpga_nconfig : out std_logic;
			
			max_led : out std_logic;
			
			image_led : out std_logic_vector(2 downto 0);
			
			srst : in std_logic;
			reset_n : in std_logic;
			fsd_req : out std_logic;
			pfl_en_out : out std_logic;
			fpga_pgm_out : out std_logic_vector(2 downto 0);
			pfl_pgm : in std_logic_vector(2 downto 0)
		);
	end component;
	signal pfl_en : std_logic;

component fppOvj is 
        port (
              -- inputs:
                 signal system_clk : in std_logic;
                 signal reset_n : in std_logic;
              
              
                 signal fpga_nstatus : IN STD_LOGIC;
                 
              -- outputs:
				signal data_out_port : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
				signal clk_out_port : out std_logic;
				signal nConfig_out_port : out std_logic
              );
end component;
	
begin

	csense_adc_f0 <= '0';
	
	max_load <= usb_led and (not fsd_req);
	
	reset_generator_inst : reset_generator port map(
			clk => clk,
			reset_in => max_resetn,
			reset => reset_n
		);	

	pgm(3 downto 0) <= (others => '0');
	
	power_inst : power port map(
			reset_n => reset_n,
			reset_vj => reset_vj,
			clk => clk,
			csense_sdo => csense_sdo,
			control_gui => control_gui,
			pgm => pgm,
			csense_sck => csense_sck,
			sense_ce0 => csense_csn(0),
			sense_ce1 => csense_csn(1),
			csense_sdi => csense_sdi,
			ave_v => ave_v,
			ave_a => ave_a,
			ave_w => ave_w,
			max_v => max_v,
			max_a => max_a,
			max_w => max_w,
			min_v => min_v,
			min_a => min_a,
			min_w => min_w,
			rail_sel_out => rail_sel_out,
--			adc_code_mon : out std_logic_vector(23 downto 0);
--			sgl_mon : out std_logic;
			power_disable => '0'
		);
		
	vj_block_inst : vj_block port map(
			reset_n => reset_n,
			ave_v => ave_v,
			ave_a => ave_a,
			ave_w => ave_w,
			max_v => max_v,
			max_a => max_a,
			max_w => max_w,
			min_v => min_v,
			min_a => min_a,
			min_w => min_w,
			temp_data_r => (others => '0'),
			temp_data_l => (others => '0'),
			volt_data => (others => '0'),
			pmon_rails => pmon_rails,
			control_gui => control_gui,
			reset_vj => reset_vj,
			MAX_VER => MAX_VER
		);

	fpga_interface_inst : fpga_interface port map(
		reset_n => reset_n,
		fsm_clk => max2_clk,
		
		--FPGA interface
		max_csn => max2_csn,
		max_wen => max2_wen,
		max_oen => max2_oen,
		max_ben => max2_ben(3 downto 0),
		
		address => fsm_a(6 downto 2),
		data => fsm_d(15 downto 0),
		
		--dipswitch/PFL interface
		dsw_pagesel => fpga_pgm,
		pfl_pagesel => pfl_pgm,
		srst_out => srst,
		
		-- power and temp data
		ave_v => ave_v,
		ave_a => ave_a,
		ave_w => ave_w,
		max_v => max_v,
		max_a => max_a,
		max_w => max_w,
		min_v => min_v,
		min_a => min_a,
		min_w => min_w,
		temp_data_r => (others => '0'),
		temp_data_l => (others => '0'),
		volt_data => (others => '0'),
		fsd_req => fsd_req,
		
		-- clk control
		clk155_en => clk155_oe,
		clk1_ce => clk1_ce,
		clk2_ce => clk2_ce,
		clk_sel => clk_sel,
		clk_enable => clk_enable,
		
		-- board setting
		sram_mode => sram_mode,
		sram_zz => sram_zz,
		hsma_orsntn => hsma_psnt_n,
--		hsmb_orsntn => hsmb_psnt_n,		--no hsmb in A2GX125
		MAX_VER => MAX_VER,

	
		-- OSC control
		CLK1_PR => CLK1_PR,
		CLK1_OD => CLK1_OD,
		CLK2_PR => CLK2_PR,
		CLK2_OD => CLK2_OD,
		clk1_rstn => clk1_rstn,
		clk2_rstn => clk2_rstn,
		
		system_clk => clk					
		);

	err_led_inst : err_led port map(
			reset_n => reset_n,
			clk => clk,
			pfl_en => pfl_en,
			conf_done => fpga_conf_done,
			error_led => max_error
		);		

--	fppOvj_inst : fppOvj port map(
--		system_clk => clk,
--		reset_n => reset_n,
--		fpga_nstatus => fpga_nstatus,
--		data_out_port => fpga_config_d,
--		clk_out_port => fpga_dclk,
--		nConfig_out_port => fpga_nconfig
--	);
				  
	pfl_control_inst : pfl_control port map(
			fpga_conf_done => fpga_conf_done,
			fpga_nstatus => fpga_nstatus,
			clk => clk,
			load_image => load_image,
			image_sel_in => image_sel,
			factory_user => factory_user,
			
			fsm_d => fsm_d(15 downto 0),
			fsm_a(24 downto 0) => fsm_a(25 downto 1),
			flash_cen => flash_cen,
			flash_wen => flash_wen,
			flash_oen => flash_oen,
			flash_clk => flash_clk,
			flash_advn => flash_advn,
			flash_resetn => flash_resetn,
			
			fpga_config_d => fpga_config_d,
			fpga_dclk => fpga_dclk,
			fpga_nconfig => fpga_nconfig,
			
			max_led => max_led,
			
			image_led => image_led,
			
			srst => srst,
			reset_n => reset_n,
			fsd_req => fsd_req,
			pfl_en_out => pfl_en,
			fpga_pgm_out => fpga_pgm,
			pfl_pgm => pfl_pgm
		);
		
end rtl;