library ieee;
use ieee.std_logic_1164.all;

entity vj_block is
	port (
		reset_n: in std_logic;
		ave_v, ave_a, ave_w : in std_logic_vector(15 downto 0);
		max_v, max_a, max_w : in std_logic_vector(15 downto 0);
		min_v, min_a, min_w : in std_logic_vector(15 downto 0);
		temp_data_r : in std_logic_vector(15 downto 0);
		temp_data_l : in std_logic_vector(15 downto 0);
		volt_data : in std_logic_vector(15 downto 0);
		pmon_rails : in std_logic_vector(3 downto 0);
--		remote_local_sel : in std_logic;
--		power_temp_sel : in std_logic;
--		gui_control : in std_logic;

		control_gui : out std_logic_vector(7 downto 0);
		reset_vj: out std_logic;
		MAX_VER : in std_logic_vector(7 downto 0)
	);
end vj_block;


architecture rtl of vj_block is
--	constant MAX2version: std_logic_vector(7 downto 0) := x"06";

	component data_enc
		port (
			clk : IN STD_LOGIC;
			cdr_in : IN STD_LOGIC;
			ir_in : IN std_logic_vector(1 downto 0);
			vj_add : IN std_logic_vector(3 downto 0);
			ave_v, ave_a, ave_w, max_v, max_a, max_w, min_v, min_a, min_w: in std_logic_vector(15 downto 0);
			temp_data_r : in std_logic_vector(15 downto 0);
			temp_data_l : in std_logic_vector(15 downto 0);
			volt_data : in std_logic_vector(15 downto 0);
			reg : in std_logic_vector(15 downto 0);

			disp_out : OUT STD_LOGIC
		  );
	end component;
	
	
	component vj_inf
		PORT(
			ir_out		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
			tdo		: IN STD_LOGIC ;
			ir_in		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			tck		: OUT STD_LOGIC ;
			tdi		: OUT STD_LOGIC ;
			virtual_state_cdr		: OUT STD_LOGIC ;
			virtual_state_cir		: OUT STD_LOGIC ;
			virtual_state_e1dr		: OUT STD_LOGIC ;
			virtual_state_e2dr		: OUT STD_LOGIC ;
			virtual_state_pdr		: OUT STD_LOGIC ;
			virtual_state_sdr		: OUT STD_LOGIC ;
			virtual_state_udr		: OUT STD_LOGIC ;
			virtual_state_uir		: OUT STD_LOGIC 
		);
	END component;
	
	
	component address_dec
		port (
			clk : in std_logic;
			 data_in : IN STD_LOGIC;
			 ir_in : IN std_logic_vector(1 downto 0);
			 sdr_in : IN STD_LOGIC;
			 e1dr_in : in std_logic;
			reset_in : in std_logic;

			 control_out : out std_logic_vector(7 downto 0);
			 address_out : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
			reset_n : out std_logic
		  );
	end component;	

	signal tck, cdr, tdo, tdi, sdr, e1dr : std_logic;
	signal ir_in : std_logic_vector(1 downto 0);
	signal vj_add : std_logic_vector(3 downto 0);
	signal reg : std_logic_vector(15 downto 0);
	signal pmon_rails_int : std_logic_vector(2 downto 0);
	
begin

	u1: data_enc port map(tck, cdr, ir_in, vj_add, 
	ave_v, ave_a, ave_w, max_v, max_a, max_w, min_v, min_a, min_w, temp_data_r, temp_data_l, volt_data, reg, tdo);
	u2: address_dec port map(tck, tdi, ir_in, sdr, e1dr, reset_n, control_gui, vj_add, reset_vj);

	vj_inf_inst : vj_inf PORT MAP (
			ir_out	 => ir_in,
			tdo	 => tdo,
			ir_in	 => ir_in,
			tck	 => tck,
			tdi	 => tdi,
			virtual_state_cdr	 => cdr,
--			virtual_state_cir	 => ,
			virtual_state_e1dr	 => e1dr,
--			virtual_state_e2dr	 => ,
--			virtual_state_pdr	 => ,
			virtual_state_sdr	 => sdr
--			virtual_state_udr	 => ,
--			virtual_state_uir	 => 
		);

--	reg <= "00" & pmon_rails_int & remote_local_sel & power_temp_sel & gui_control & "00000011";
	reg <= '0' & pmon_rails & '0' & '0' & '0' & MAX_VER;

end rtl;